{
  Component Names Property Editor 2.2

	Copyright 1997 Roelof Ensing, all rights reserved.

	By:					Roelof Ensing
  email:      EnsingRoel@msn.com

  Just a small property editor to edit the names of the components quickly
  with a predefined prefix, suffix and/or name.

  I was getting a little bit sick of constantly having to type in the
  prefixes for a Hungarian notation of the component names, so decided
  to create a small property editor for this.

  History:

  1.0   Initial version, Q&D

  2.0   Changed the editor to use information out of the Registry instead
        out of internal tables. Plus added the update function to allow
        for interactive update of the registry instead of out of the
        PENAMES.REG file.

  2.1	  Changed the functionality for suggestions to handle custom
  		  components, see PENAMEX.PAS

  2.2   a)	Use the root key from Delphi
  			b)	For speed improvement, load on initialization the prefixlist,
        		the suffixlist and namelist in memory.

	DISCLAIMER

 	Users of this software product or component (and it's components) must
 	accept this disclaimer of	warranty:

 	This software product or component	is supplied as is.	The author
 	disclaims all warranties, expressed or implied, including, without
 	limitation,	the warranties of merchantability and of fitness for any
 	purpose.

	The author assumes no liability for damages, direct or consequential,
	which may result from the use of this software product or component.

	Software may be used, modified and distributed freely if compiled in
	with commercial or private applications (not another VCL).

	The developer retains the copyright to this Software.	You may not
	distribute the source code (PAS) or its compiled unit (DCU) for profit.

	***********************************************************************
	*                                                                     *
	*  PLEASE LET ME KNOW IF YOU DOWNLOAD THIS EDITOR AND FIND IT USEFULL *
	*  I'M INTERESTED IN TO KNOW HOW MUCH INTEREST THERE IS IN LITTLE     *
	*  GADGETS LIKE THIS. 																								*
	*                                                                     *
	*	  PLEASE FORWARD ANY COMMENTS OR SUGGESTIONS TO ROELOF ENSING AT:		*
	*                                                                     *
	*											EnsingRoel@msn.com                              *
	*                                                                     *
	***********************************************************************

}

unit penames;

interface

uses
  Windows, Messages, SysUtils, Classes, StdCtrls, Forms, Buttons, Dialogs,
  dsgnintf, Controls, ComCtrls, ExtCtrls;

type
  TComponentNameEditorForm = class(TForm)
    PageControl: TPageControl;
    SheetSetName: TTabSheet;
    SheetOptions: TTabSheet;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label1: TLabel;
    cbUpdateName: TBitBtn;
    cbUpdateClass: TBitBtn;
    edAssignedName: TEdit;
    cbxUsePrefix: TCheckBox;
    edResult: TEdit;
    cbSuggest: TBitBtn;
    cmbPrefix: TComboBox;
    cbxUseSuffix: TCheckBox;
    CmbSuffix: TComboBox;
    cmbNewName: TComboBox;
    edClassName: TEdit;
    cbApply: TBitBtn;
    cbCancel: TBitBtn;
    GroupBox1: TGroupBox;
    cbxSeperatePrefixandName: TCheckBox;
    cbxSeperateSuffixandName: TCheckBox;
    Label7: TLabel;
    Bevel1: TBevel;
    lblKey: TLabel;
    procedure FormShow(Sender: TObject);
    procedure cbApplyClick(Sender: TObject);
    procedure edNewNameKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cbSuggestClick(Sender: TObject);
    procedure BuildResultName(Sender: TObject);
    procedure UpdateClassClick(Sender: TObject);
    procedure cbUpdateNameClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cbxSeperateSuffixandNameClick(Sender: TObject);
    procedure cbxSeperatePrefixandNameClick(Sender: TObject);
  private
    { Private declarations }
		procedure SaveOptions(Sender: TObject);
  public
    { Public declarations }
    CurrComp:TComponent;
    CurrName:TComponentName;
  end;

  TPrefixedNameEditor = Class(TPropertyEditor)
  public
    function  GetValue: String; override;
    procedure SetValue(Const Value: string); override;
    function  GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

var
  ComponentNameEditorForm: TComponentNameEditorForm;

	function SetComponentName(AComponent:TComponent;const AName:String):string;

implementation

uses menus, registry, exptintf, toolintf, penamesx;

{$R *.DFM}

const
	sRegBase				=
	{$IFDEF VER90}	'Software\Borland\Delphi\2.0';
	{$ELSE}
  {$IFDEF VER100}	'Software\Borland\Delphi\3.0';
	{$ELSE}
  {$IFDEF VER120}	'Software\Borland\Delphi\4.0';
	{$ELSE}					'Software\Borland\Delphi';
	{$ENDIF}
  {$ENDIF}
  {$ENDIF}
	sRegEditors   			= '\Property Editors';
  sRegNameEditor  		= '\ComponentName Editor';
	sRegPrefix					= '\Prefix';
	sRegSuffix					= '\Suffix';
	sRegDefault		 			= '\Default';
  sRegPrefixSeperator	=	'UsePrefixSeperator';
  sRegSuffixSeperator	=	'UseSuffixSeperator';

	sCustom					=	'Custom';
	sForm						=	'Form';
	sDialog					=	'Dialog';

var
  { Static info, created and loaded on initialization }
	Regis				: TRegistry;
	RegBase			: string;
  RegKey 			: string;
	RegPrefix		: string;
	RegSuffix		: string;
	RegDefault 	: string;
	PrefixText  : string;
  PrefixList  : TStringList;
  SuffixList  : TStringList;
  NameList    : TStringList;

{ =================================================================== }
{ Compose the registry keys 																					}
{ =================================================================== }

function SetRegistryKeys:string;
var
  FTS:TIToolServices;
  BS:string;
begin
	BS:='';
	try
		FTS := ExptIntf.ToolServices;
  	BS  := Trim(FTS.GetBaseRegistryKey);
  except
  	BS:='';
  end;
  if BS <> '' then
  	RegBase:=BS
  else
  	RegBase:=sRegBase;
  { Avoid any double slashes }
  while (Length(BS) > 0)
  and (BS[Length(BS)] = '\') do
  	SetLength(BS,Length(BS)-1);
	RegKey 			:= RegBase + sRegEditors + sRegNameEditor;
	RegPrefix		:= RegKey  + sRegPrefix;
	RegSuffix		:= RegKey  + sRegSuffix;
	RegDefault 	:= RegKey  + sRegDefault;
  Result:=RegBase;
end;

{ =================================================================== }
{ Make sure the name is valid																					}
{ =================================================================== }

procedure CheckForValidName(var AValue:string);
var
	i:longint;
	s:string;
begin
	s:='';
	if Length(AValue) < 1 then exit;
	for i:=length(AValue) downto  1 do
		if AValue[i] in ['0'..'9','A'..'Z','a'..'z','_']  then
			s:=AValue[i]+s;
	AValue:=s;
end;

{ =================================================================== }
{ Remove any numbers at the end of the name 													}
{ =================================================================== }

procedure CheckForNumbersAtEnd(var AValue:string);
var
	i:longint;
begin
	if Length(AValue) < 1 then
		exit;
	for i:=length(AValue) downto 1 do
		if AValue[i] in ['0'..'9']  then
			SetLength(AValue,Length(AValue)-1)
		else
			break;
end;

{ =================================================================== }
{ Just to get to the caption property 			 													}
{ =================================================================== }

type
	TMyControl = class(TWinControl)
  	protected
    	property Caption;
  end;


{ =================================================================== }
{ Get a suggested name using caption, text, etc												}
{ =================================================================== }

function GetSuggestion(AComponent:TComponent):string;
begin

  Result:=GetCustomSuggestion(AComponent);

  if Result = '' then
  begin
 		if AComponent is TWinControl then
    	Result:=TMyControl(AComponent).Caption
    else
 			Result:=AComponent.Name;
 	end;

  CheckForValidName(Result);

  CheckForNumbersAtEnd(Result);

  if CompareStr(PrefixText,Copy(Result,1,Length(PrefixText))) = 0 then
  	Result:=Copy(	Result,
  								Length(PrefixText)+1,
  								Length(Result)-Length(PrefixText));
end;

{ =================================================================== }
{ Get the class prefixes out of the registry   												}
{ =================================================================== }

procedure GetClassPrefixList(AList:TStrings);
var
	i:integer;
begin
	AList.Clear;
  Regis.OpenKey(RegPrefix,true);
  Regis.GetValueNames(AList);
  for i:=0 to AList.Count -1 do
  	AList[i]:=Regis.ReadString(AList[i]);
  for i:=AList.Count -1 downto 0 do
  begin
  	if AList[i] = '' then
  		AList.delete(i)
  	else
  	if AList.IndexOf(AList[i]) <> i then
  		AList.delete(i);
  end;
  Regis.CloseKey;
end;

{ =================================================================== }
{ Get the class suffixes out of the registry   												}
{ =================================================================== }

procedure GetClassSuffixList(AList:TStrings);
var
	i:integer;
begin
	AList.Clear;
  Regis.OpenKey(RegSuffix,true);
  Regis.GetValueNames(AList);
  for i:=0 to AList.Count -1 do
  	AList[i]:=Regis.ReadString(AList[i]);
  for i:=AList.Count -1 downto 0 do
  begin
  	if AList[i] = '' then
  		AList.delete(i)
  	else
  	if AList.IndexOf(AList[i]) <> i then
  		AList.delete(i);
  end;
  Regis.CloseKey;
end;

{ =================================================================== }
{ Get the class speficic prefix                												}
{ =================================================================== }

function GetClassPrefix(AClass:TClass):string;
begin
	Result:='';
	Regis.OpenKey(RegPrefix,true);
	Result:=Regis.ReadString(AClass.ClassName);
	PrefixText:=Result;
	Regis.CloseKey;
end;

{ =================================================================== }
{ Get the class speficic suffix                												}
{ =================================================================== }

function GetClassSuffix(AClass:TClass):string;
begin
	Result:='';
	Regis.OpenKey(RegSuffix,true);
	Result:=Regis.ReadString(AClass.ClassName);
	Regis.CloseKey;
end;

{ =================================================================== }
{ Get the components prefix                    												}
{ =================================================================== }

function GetComponentPrefix(AComponent:TComponent):string;
var
	CheckClass:TCLass;
begin
	Result:='';
	CheckClass:=AComponent.ClassType;
	while ( CheckClass <> nil ) and (Result = '') do
	begin
		Result:=GetClassPrefix(CheckClass);
    CheckClass:=CheckClass.ClassParent;
	end;
	if Result = '' then
		Result := sCustom;
	PrefixText:=Result;
end;

{ =================================================================== }
{ Get the components suffix                    												}
{ =================================================================== }

function GetComponentSuffix(AComponent:TComponent):string;
var
	CheckClass:TCLass;
begin
	Result:='';

	{ Handle a form seperate }

	if ( AComponent is TForm ) then
	begin
		if TForm(AComponent).BorderStyle = bsDialog then
			Result := sDialog
		else
			Result := sForm;
		exit;
	end;

	CheckClass:=AComponent.ClassType;

	while ( CheckClass <> nil ) and (Result = '') do
	begin
		Result:=GetClassSuffix(CheckClass);
    CheckClass:=CheckClass.ClassParent;
	end;

	if ( AComponent is TMenuItem ) then
	begin
		if TComponent(TMenuItem(AComponent).Parent) is TPopupMenu then
			Result:='p'+Result;
	end;

end;

{ =================================================================== }
{ Get the component's complete name with prefix 											}
{ =================================================================== }

function GetClassPrefixedName(AComponent:TComponent;const AName:TComponentName):string;
begin
	Result:=AName;
  Result:=GetComponentPrefix(AComponent)+AName;
end;

{ =================================================================== }
{ Show the editor dialog 				     																	}
{ =================================================================== }

function SetComponentName(AComponent:TComponent;const AName:String):string;
begin
	Result:=AName;
	with TComponentNameEditorForm.Create(nil) do
  begin
  	CurrName:=AName;
    CurrComp:=AComponent;
  	if ShowModal = mrOK then
  		Result:=CurrName;
    Release;
  end;
end;

{ ========================================================================= }
{ Load the information Lists}
{ ========================================================================= }

procedure LoadStaticInfo;
begin
	PrefixList:=TStringList.Create;
  if PrefixList = nil then
  	EInvalidOperation.Create('Name Editor : Error 1 : Prefixes, Out of memory');
  SuffixList:=TStringList.Create;
  if SuffixList = nil then
  	EInvalidOperation.Create('Name Editor : Error 2 : Suffixes, Out of memory');
  NameList:=TStringList.Create;
  if NameList = nil then
  	EInvalidOperation.Create('Name Editor : Error 3 : Names, Out of memory');
	Regis:=TRegistry.Create;
  if Regis = nil then
  	EInvalidOperation.Create('Name Editor : Error 4 : Registry, Out of memory');
	SetRegistryKeys;
  GetClassPrefixList(PrefixList);
  GetClassSuffixList(SuffixList);
  Regis.CloseKey;
  Regis.OpenKey(RegDefault,true);
  Regis.GetValueNames(NameList);
  Regis.CloseKey;
  PrefixList.Sorted:=true;
  PrefixList.Duplicates:=dupIgnore;
  SuffixList.Sorted:=true;
  SuffixList.Duplicates:=dupIgnore;
  NameList.Sorted:=true;
  NameList.Duplicates:=dupIgnore;
end;

{ ========================================================================= }
{ Release the information lists }
{ ========================================================================= }

procedure DestroyStaticInfo;
begin
	try
		PrefixList.Free;
  	SuffixList.Free;
  	NameList.Free;
  	Regis.Free;
  finally
  	;
  end;
end;

{ ========================================================================= }
{ The Property Editor }
{ ========================================================================= }

function TPrefixedNameEditor.GetAttributes: TPropertyAttributes;
begin
  Result := [ paDialog ];
end;

procedure TPrefixedNameEditor.Edit;
var
  UseComp:TComponent;
  UseName:TComponentName;
begin
 	//Regis	:= TRegistry.Create;
 	try
 		UseName	:= GetValue;
   	UseComp	:= TComponent(GetComponent(0));
   	SetValue(SetComponentName(UseComp,UseName));
 	finally
   	//Regis.Free;
    ;
  end;
end;

function TPrefixedNameEditor.GetValue: string;
begin
  Result := GetStrValue;
end;

procedure TPrefixedNameEditor.SetValue(const Value : string);
begin
	SetStrValue(Value);
end;

{ =================================================================== }
{ The editors dialog    																							}
{ =================================================================== }

procedure TComponentNameEditorForm.FormShow(Sender: TObject);
begin
  if Regis = nil then
  	EInvalidOperation.Create('Name Editor : Error 5 : Form Show, Out of memory');  SetRegistryKeys;
	PageControl.ActivePage:=SheetSetName;
  lblKey.Caption:=RegKey;
  lblKey.Hint:=lblKey.Caption;
	with Regis do
  begin
		OpenKey(RegKey,true);
    try
    	if ValueExists(sRegPrefixSeperator) then
    		cbxSeperatePrefixandName.Checked:=ReadBool(sRegPrefixSeperator);
	    if ValueExists(sRegSuffixSeperator) then
  	  	cbxSeperateSuffixandName.Checked:=ReadBool(sRegSuffixSeperator);
    finally
    	;
    end;
    CloseKey;
  end;
  if CurrComp=nil then
  	CurrComp:=SELF;
	edClassName.Text:=CurrComp.ClassName;
	edAssignedName.text := CurrName;
	cmbPrefix.Items.Assign(PrefixList);//GetClassPrefixList(cmbPrefix.Items);
  cmbSuffix.Items.Assign(SuffixList);//GetClassSuffixList(cmbSuffix.Items);
  cmbPrefix.Text:=GetComponentPrefix(CurrComp);
  cmbSuffix.Text:=GetComponentSuffix(CurrComp);
  Regis.CloseKey;
  Regis.OpenKey(RegDefault,true);
  cmbNewName.Items.Assign(NameList);//Regis.GetValueNames(cmbNewName.Items);
  Regis.CloseKey;
  cmbNewName.Text:='';
	BuildResultName(SELF);
  cmbNewName.SetFocus;
  cbxUsePrefix.Checked := CompareText(cmbPrefix.Text,sCustom) <> 0;
  cbxUseSuffix.Checked := CompareText(cmbSuffix.Text,sCustom) <> 0;
end;

procedure TComponentNameEditorForm.BuildResultName(Sender: TObject);
var
	s1,s2:string;
begin
	s1:='';
	if cbxUsePrefix.checked then
  begin
  	s1:=Trim(cmbPrefix.Text);
    if s1 <> '' then
     	if cbxSeperatePrefixandName.checked then
  			AppendStr(s1,'_');
  end;
	s2:=cmbNewName.Text;
	if Length(s2) > 0 then
		s2[1]:=UpCase(s2[1]);
  AppendStr(s1,s2);
	if cbxUseSuffix.checked then
  begin
  	if Trim(cmbSuffix.Text) <> '' then
  	begin
    	if cbxSeperateSuffixandName.checked then
  			AppendStr(s1,'_');
  		AppendStr(s1,Trim(cmbSuffix.Text));
    end;
  end;
  edResult.Text:=s1;
end;

procedure TComponentNameEditorForm.cbApplyClick(Sender: TObject);
begin
	if edResult.Text <> '' then
	begin
  	CurrName:=edResult.Text;
  	ModalResult:=mrOK;
  end
  else
  	ShowMessage('Result can not be empty');
end;

procedure TComponentNameEditorForm.edNewNameKeyUp(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
	BuildResultName(SELF);
end;

procedure TComponentNameEditorForm.cbSuggestClick(Sender: TObject);
var
	s:string;
begin
  s:=GetSuggestion(CurrComp);
  if s <> '' then
   	cmbNewName.Text := s;
 BuildResultName(SELF);
end;

procedure TComponentNameEditorForm.UpdateClassClick(Sender: TObject);
begin
	{ Update the registry }
  try
  	with Regis do
    begin
      if cbxUsePrefix.Checked then
      begin
      	OpenKey(RegPrefix,true);
        WriteString(CurrComp.ClassName,cmbPrefix.Text);
     	 	PrefixList.Add(cmbPrefix.Text);
      	CloseKey;
      end;
      if cbxUseSuffix.Checked then
      begin
    		OpenKey(RegSuffix,true);
      	WriteString(CurrComp.ClassName,cmbSuffix.Text);
      	SuffixList.Add(cmbSuffix.Text);
      	CloseKey;
      end;
    end;
  finally
  	Regis.CloseKey;
  end;

end;

procedure TComponentNameEditorForm.cbUpdateNameClick(Sender: TObject);
begin
	{ Update the registry }
  try
  	with Regis do
    begin
      if cmbNewName.Text <> '' then
      begin
    		OpenKey(RegDefault,true);
      	WriteString(cmbNewName.Text,'');
        NameList.Add(cmbNewName.Text);
      	CloseKey;
      end;
    end;
  finally
  	Regis.CloseKey;
  end;
end;

procedure TComponentNameEditorForm.FormCreate(Sender: TObject);
begin
  if Regis = nil then
  begin
  	LoadStaticInfo;
  end;
end;

procedure TComponentNameEditorForm.FormDestroy(Sender: TObject);
begin
  //DestroyStaticInfo;
end;

procedure TComponentNameEditorForm.SaveOptions(Sender: TObject);
begin
	with Regis do
  begin
		OpenKey(RegKey,true);
    with cbxSeperatePrefixandName do
    	WriteBool(sRegPrefixSeperator,Checked);
    with cbxSeperateSuffixandName do
    	WriteBool(sRegSuffixSeperator,Checked);
    CloseKey;
  end;
end;

procedure TComponentNameEditorForm.cbxSeperateSuffixandNameClick(
  Sender: TObject);
begin
	SaveOptions(SELF);
  BuildResultname(SELF);
end;

procedure TComponentNameEditorForm.cbxSeperatePrefixandNameClick(
  Sender: TObject);
begin
	SaveOptions(SELF);
  BuildResultname(SELF);
end;

initialization
	LoadStaticInfo;

finalization
	DestroyStaticInfo;

end.
